/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal.introspection.details.handler;

import org.conqat.ide.commons.ui.jface.TreeContentProviderBase;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ITableLabelProvider;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.TableLabelProviderBase;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.ui.dialogs.PatternFilter;
import org.fortiss.tooling.kernel.extension.IStorageProvider;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.extension.data.ModelStorageError;
import org.fortiss.tooling.kernel.introspection.items.PersistencyServiceIntrospectionDetailsItem;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.ui.service.IModelElementHandlerService;

/**
 * Introspection UI handler for the {@link IPersistencyService}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 7F091F376579C57699B5DD86596A6848
 */
public final class PersistencyServiceIntrospectionDetailsUIHandler extends
		IntrospectionDetailsUIHandlerBase {

	/** {@inheritDoc} */
	@Override
	public Control createComposite(ScrolledComposite parent) {
		String heading = "Type to search registered persistency providers:";
		String footer =
				"Number of currently registered providers: " +
						getInputObject().getStorageProviders().size();
		return createFilteredTreeInTabFolder(parent, heading, footer, "Registered Providers");
	}

	/** {@inheritDoc} */
	@Override
	protected void createTreeColumns(Tree tree) {
		TreeColumn col0 = new TreeColumn(tree, SWT.LEFT);
		col0.setText("Storage Provider Class / Toplevel Element");
		col0.setWidth(450);

		TreeColumn col1 = new TreeColumn(tree, SWT.LEFT);
		col1.setText("Toplevel EObject Class / Storage Error Message");
		col1.setWidth(400);

		TreeColumn col2 = new TreeColumn(tree, SWT.LEFT);
		col2.setText("Error Cause Exception");
		col2.setWidth(400);
	}

	/** {@inheritDoc} */
	@Override
	protected ITreeContentProvider createContentProvider() {
		return new TreeContentProviderBase() {

			@Override
			public Object[] getChildren(Object parentElement) {
				if(parentElement == getInputObject()) {
					return getInputObject().getStorageProviders().toArray();
				}
				if(parentElement instanceof IStorageProvider) {
					IStorageProvider prov = (IStorageProvider)parentElement;
					return new Object[] {new ToplevelOrErrorTreeNode(prov, true),
							new ToplevelOrErrorTreeNode(prov, false)};
				}
				if(parentElement instanceof ToplevelOrErrorTreeNode) {
					ToplevelOrErrorTreeNode node = (ToplevelOrErrorTreeNode)parentElement;
					if(node.isToplevel) {
						return node.provider.getTopLevelElements().toArray();
					}
					return node.provider.getStorageErrors().toArray();
				}
				return null;
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	protected ITableLabelProvider createLabelProvider() {
		return new TableLabelProviderBase() {
			@Override
			public String getColumnText(Object parentElement, int columnIndex) {
				if(parentElement instanceof IStorageProvider) {
					IStorageProvider isp = (IStorageProvider)parentElement;
					return columnIndex == 0 ? isp.getClass().getName() : "";
				}
				if(parentElement instanceof ToplevelOrErrorTreeNode) {
					ToplevelOrErrorTreeNode node = (ToplevelOrErrorTreeNode)parentElement;
					if(node.isToplevel) {
						return columnIndex == 0 ? "Toplevel Elements" : "";
					}
					return columnIndex == 0 ? "Model Storage Errors" : "";
				}
				if(parentElement instanceof ITopLevelElement) {
					ITopLevelElement top = (ITopLevelElement)parentElement;
					switch(columnIndex) {
						case 0:
							return IModelElementHandlerService.getInstance().getName(top
									.getRootModelElement());
						case 1:
							return top.getRootModelElement().getClass().getName();
						default:
							return "";
					}
				}
				if(parentElement instanceof ModelStorageError) {
					ModelStorageError error = (ModelStorageError)parentElement;
					switch(columnIndex) {
						case 0:
							return "";
						case 1:
							return error.getMessage();
						case 2:
							return error.getCause().toString();
					}
				}
				return "";
			}
		};
	}

	/** {@inheritDoc} */
	@Override
	protected boolean testSelection(Object selection) {
		return selection instanceof IStorageProvider || selection instanceof ITopLevelElement;
	}

	/** {@inheritDoc} */
	@Override
	protected void populateContextMenu(Object selection, MenuManager mgr) {
		if(selection instanceof IStorageProvider) {
			mgr.add(createCopyClassNameAction(selection.getClass()));
		} else if(selection instanceof ITopLevelElement) {
			EObject eo = ((ITopLevelElement)selection).getRootModelElement();
			mgr.add(createCopyClassNameAction(eo.getClass()));
		}
	}

	/** {@inheritDoc} */
	@Override
	protected PatternFilter createPatternFilter() {
		return new TableViewerPatternFilter();
	}

	/** {@inheritDoc} */
	@Override
	protected PersistencyServiceIntrospectionDetailsItem getInputObject() {
		return (PersistencyServiceIntrospectionDetailsItem)dataItem;
	}

	/** Class for tree nodes below storage providers. */
	private static class ToplevelOrErrorTreeNode {
		/** The storage provider. */
		public final IStorageProvider provider;
		/** The toplevel flag. */
		public final boolean isToplevel;

		/** Constructor. */
		public ToplevelOrErrorTreeNode(IStorageProvider provider, boolean toplevel) {
			this.provider = provider;
			this.isToplevel = toplevel;
		}
	}
}
