/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.util;

import static org.fortiss.tooling.kernel.extension.data.IConstraintViolation.SEVERITY_DIRECT_COMPARATOR;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.widgets.Display;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation;
import org.fortiss.tooling.kernel.extension.data.IConstraintViolation.ESeverity;
import org.fortiss.tooling.kernel.model.constraints.ConstrainedWithChecksum;
import org.fortiss.tooling.kernel.model.constraints.ErrorVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.FailVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.IConstrained;
import org.fortiss.tooling.kernel.model.constraints.IConstraint;
import org.fortiss.tooling.kernel.model.constraints.IConstraintVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.OutdatedVerificationStatus;
import org.fortiss.tooling.kernel.model.constraints.SuccessVerificationStatus;
import org.fortiss.tooling.kernel.service.IConstraintVerificationService;
import org.fortiss.tooling.kernel.ui.service.IConstraintVerificationUIService;
import org.fortiss.tooling.kernel.utils.EcoreUtils;

/**
 * Utility functions for constraints.
 * 
 * @author aravantinos
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: BC06655274768325481A442F27DF78E6
 */
public class ConstraintsUIUtils {

	/** Gathers some statuses by verification status. */
	public static class ClassifiedStatuses {

		/** Failed constraints. */
		public List<FailVerificationStatus> failedStatuses;

		/** Error statuses (constraint could not be checked). */
		public List<ErrorVerificationStatus> errorStatuses;

		/** Outdated statuses. */
		public List<OutdatedVerificationStatus> outdatedStatuses;

		/** Successful constraints. */
		public List<SuccessVerificationStatus> successStatuses;

		/** Constructor. */
		public ClassifiedStatuses() {
			failedStatuses = new ArrayList<FailVerificationStatus>();
			errorStatuses = new ArrayList<ErrorVerificationStatus>();
			outdatedStatuses = new ArrayList<OutdatedVerificationStatus>();
			successStatuses = new ArrayList<SuccessVerificationStatus>();
		}
	}

	/**
	 * @param constrained
	 * @return {@link ClassifiedStatuses} corresponding to <code>constrained</code>.
	 */
	public static ClassifiedStatuses getClassifiedStatuses(IConstrained constrained) {
		ClassifiedStatuses res = new ClassifiedStatuses();
		for(IConstraint constraint : constrained.getConstraints()) {
			IConstraintVerificationStatus status = constraint.getVerificationStatus();
			if(status instanceof FailVerificationStatus) {
				res.failedStatuses.add((FailVerificationStatus)status);
			}
			if(status instanceof ErrorVerificationStatus) {
				res.errorStatuses.add((ErrorVerificationStatus)status);
			}
			if(status instanceof OutdatedVerificationStatus) {
				res.outdatedStatuses.add((OutdatedVerificationStatus)status);
			}
			if(status instanceof SuccessVerificationStatus) {
				res.successStatuses.add((SuccessVerificationStatus)status);
			}
		}
		return res;
	}

	/**
	 * @param c
	 * @return {@link ESeverity} corresponding to the status of the constraint <code>c</code>
	 */
	public static ESeverity getConstraintSeverity(IConstraint c) {
		if(c.getVerificationStatus() instanceof FailVerificationStatus) {
			return ESeverity.ERROR;
		} else if(c.getVerificationStatus() instanceof ErrorVerificationStatus) {
			return ESeverity.ERROR;
		} else if(c.getVerificationStatus() instanceof OutdatedVerificationStatus) {
			return ESeverity.WARNING;
		} else {
			return ESeverity.lowest();
		}
	}

	/**
	 * @param c
	 * @return Severity corresponding to the status of the constraints of <code>c</code>.
	 */
	public static ESeverity getSeverity(IConstrained c) {
		try {
			return c.getConstraints().stream().map(cstr -> getConstraintSeverity(cstr))
					.sorted(IConstraintViolation.SEVERITY_DIRECT_COMPARATOR).findFirst().get();
		} catch(Exception e) {
			return null;
		}
	}

	/**
	 * @param sev
	 * @param modelElement
	 * @return the maximum severity between <code>sev</code> and the severity resulting of possibly
	 *         unsatisfied constraints on <code>modelElement</code>.
	 * 
	 *         Note: if, later on, {@link IConstraintViolation}
	 */
	public static ESeverity augmentSeverityWithConstraintSeverity(ESeverity sev,
			EObject modelElement) {
		if(modelElement instanceof IConstrained) {
			ESeverity sev2 = ConstraintsUIUtils.getSeverity((IConstrained)modelElement);
			if(sev == null || (sev2 != null && SEVERITY_DIRECT_COMPARATOR.compare(sev2, sev) < 0)) {
				sev = sev2;
			}
		}
		return sev == null ? ESeverity.lowest() : sev;
	}

	/**
	 * @param c
	 *            Sends (kernel-specific) refresh notifications to the constrained elements to
	 *            trigger a refresh of the marker decorations.
	 */
	public static void triggerMarkersRefresh(IConstraint c) {
		List<IConstrained> constraineds = new ArrayList<IConstrained>();
		// We first collect separately the constrained objects because - unfortunately for some
		// unknown reason - the following notification seems to trigger a change of the list of
		// constrained objects.
		for(ConstrainedWithChecksum cwc : c.getConstrainedsWithChecksum()) {
			constraineds.add(cwc.getConstrained());
		}
		constraineds.stream().forEach(
				constrained -> EcoreUtils.postRefreshNotification(constrained));
	}

	/**
	 * @param c
	 * @return Standard text describing the status of <code>c</code>.
	 */
	public static String getText(IConstraint c) {
		if(c == null) {
			return "";
		}
		IConstraintVerificationStatus status = c.getVerificationStatus();
		String mainMsg = "ERROR";
		if(status instanceof SuccessVerificationStatus) {
			mainMsg = "SUCCESS";
		} else if(status instanceof FailVerificationStatus) {
			mainMsg = "FAIL";
		} else if(status instanceof OutdatedVerificationStatus) {
			mainMsg = "OUTDATED";
		}
		return mainMsg;
	}

	/**
	 * @param c
	 * @return Standard "hint" indicating possible action on the status of <code>c</code>.
	 */
	public static String getHint(IConstraint c) {
		if(c == null) {
			return "";
		}
		IConstraintVerificationStatus status = c.getVerificationStatus();
		if(status instanceof OutdatedVerificationStatus) {
			return "(double-click to update)";
		} else if(IConstraintVerificationUIService.getInstance().canOpen(status)) {
			return "(double-click for more details)";
		}
		return "";
	}

	/**
	 * @param c
	 * @return Standard colour corresponding to the status of <code>c</code>.
	 */
	public static Color getColor(IConstraint c) {
		Display display = Display.getCurrent();
		if(c == null) {
			return display.getSystemColor(SWT.COLOR_GRAY);
		}
		IConstraintVerificationStatus status = c.getVerificationStatus();
		if(status instanceof FailVerificationStatus || status instanceof ErrorVerificationStatus) {
			return display.getSystemColor(SWT.COLOR_RED);
		} else if(status instanceof SuccessVerificationStatus) {
			return display.getSystemColor(SWT.COLOR_GREEN);
		}
		return display.getSystemColor(SWT.COLOR_GRAY);
	}

	/** A label provider for the status of an {@link IConstraint}. */
	public static class StatusLabelProvider extends ColumnLabelProvider {

		/** See constructor. */
		private boolean withHint;

		/** Function to retrieve a constraint from the embedded objects. */
		private Function<Object, IConstraint> getConstraint;

		/**
		 * Constructor.
		 * 
		 * @param withHint
		 *            <code>true</code> if you want to display hints like "double-click for more
		 *            details" in the status. If you set it to <code>true</code>, you are
		 *            responsible for providing the corresponding behaviour (e.g., that
		 *            double-clicking will open the status).
		 * @param getConstraint
		 *            Function returning a constraint from the object to be provided.
		 */
		public StatusLabelProvider(boolean withHint, Function<Object, IConstraint> getConstraint) {
			super();
			this.getConstraint = getConstraint;
			this.withHint = withHint;
		}

		/** {@inheritDoc} */
		@Override
		public String getText(Object element) {
			IConstraint cstr = getConstraint.apply(element);
			String mainMsg = ConstraintsUIUtils.getText(cstr);
			return mainMsg + (withHint ? " " + ConstraintsUIUtils.getHint(cstr) : "");
		}

		/** {@inheritDoc} */
		@Override
		public Color getBackground(Object element) {
			IConstraint cstr = getConstraint.apply(element);
			return ConstraintsUIUtils.getColor(cstr);
		}
	}

	/**
	 * @param c
	 * @param value
	 *            Sets the <code>autoCheck</code> field of <code>c</code> to <code>value</code>
	 *            <b>and triggers the verification of <code>c</code> if <code>value</code> is
	 *            <code>true</code>.</b>
	 *            Use this instead of directly setting <code>autocheck</code>.
	 */
	public static void setAutoCheck(IConstraint c, boolean value) {
		c.setAutoCheck(value);
		if(value) {
			Display display = Display.getDefault();
			Job job = new Job("Constraint verification job") {
				@Override
				protected IStatus run(IProgressMonitor monitor) {
					synchronized(c) {
						IConstraintVerificationService.getInstance().verify(c);
					}
					display.asyncExec(() -> ConstraintsUIUtils.triggerMarkersRefresh(c));
					return Status.OK_STATUS;
				}
			};
			job.schedule();
		}
	}
}
