/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2015 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal;

import java.util.Collections;
import java.util.List;

import org.fortiss.tooling.kernel.extension.LogMessageHandler;
import org.fortiss.tooling.kernel.extension.data.ILogMessageContext;
import org.fortiss.tooling.kernel.extension.data.LogMessage;
import org.fortiss.tooling.kernel.introspection.KernelIntrospectionSystemService;
import org.fortiss.tooling.kernel.service.ILoggingService;
import org.fortiss.tooling.kernel.service.base.ObjectAwareServiceBase;

/**
 * Implements the {@link ILoggingService}. It ensures that the matching handlers are called.
 * 
 * @author diewald
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 9ECDE82C4BB6B22507A6AF8DA22C6A70
 */
public class LoggingService extends ObjectAwareServiceBase<LogMessageHandler> implements
		ILoggingService {

	/** The UI message provider extension point ID. */
	private static final String EXTENSION_POINT_NAME =
			"org.fortiss.tooling.kernel.uiMessageHandler";

	/** The UI message provider configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "uiMessageHandler";

	/** The UI message provider attribue's element name. */
	private static final String ATTRIBUTE_NAME = "handler";

	/** {@inheritDoc} */
	@Override
	public void startService() {
		KernelIntrospectionSystemService.getInstance().registerService(this);
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionDescription() {
		// TODO (FH): implement
		return "";
	}

	/** {@inheritDoc} */
	@Override
	public String getIntrospectionLabel() {
		return "Logging Service";
	}

	/** {@inheritDoc} */
	@Override
	public void processMessage(LogMessage message, ILogMessageContext context) {
		List<LogMessageHandler> handlerList = findWorkingUIMessageHandler(message, context);

		for(LogMessageHandler handler : handlerList) {
			handler.showMessage(message);
		}
	}

	/**
	 * Returns the List of {@link LogMessageHandler}s that accept the given {@link LogMessage}.
	 */
	private List<LogMessageHandler> findWorkingUIMessageHandler(LogMessage message,
			ILogMessageContext context) {
		List<LogMessageHandler> handlerList = getRegisteredHandlers(message.getClass());
		if(handlerList == null) {
			return Collections.emptyList();
		}

		for(LogMessageHandler handler : handlerList) {
			if(!handler.acceptMessage(message, context)) {
				handlerList.remove(handler);
			}
		}

		return handlerList;
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return ATTRIBUTE_NAME;
	}
}
