/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.model.IIdLabeled;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Utility methods for unique IDs of {@link IIdLabeled} model elements.
 * IMPORTANT: Use this methods with care! It is usually the task of the persistency
 * layer to assign consistent IDs. However, test models and programmatic models
 * may need to use this capability without the persistency mechanism in place.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: C1FB52C5F3C9647F0B3A7E08A766EB60
 */
public class UniqueIDUtils {

	/**
	 * Generates missing IDs of the given model starting from the given current
	 * maximum ID.
	 * 
	 * @param existingModel
	 *            the model to be updated
	 * @param currentMaxId
	 *            the current maximum ID
	 * 
	 * @return The maximum ID after the generation of missing IDs.
	 */
	public static int generateMissingIDs(EObject existingModel, int currentMaxId) {
		for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				if(((IIdLabeled)eo).getId() <= 0) {
					((IIdLabeled)eo).setId((++currentMaxId));
				}
			}
		}
		return currentMaxId;
	}

	/**
	 * Removes duplicate IDs from the given model by setting them to {@code 0}. Subsequently,
	 * {@link #generateMissingIDs(EObject, int)} may be used to regenerate the duplicates that have
	 * been removed.
	 * 
	 * @param existingModel
	 *            the model to be updated
	 * @return
	 *         {@code true} if the model contained at least one duplicate that has been removed.
	 */
	public static boolean removeDuplicateIds(EObject existingModel) {
		Set<Integer> ids = new HashSet<Integer>();
		boolean removedDuplicateId = false;

		for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				IIdLabeled element = (IIdLabeled)eo;
				int id = element.getId();
				if(ids.contains(id)) {
					element.setId(0);
					removedDuplicateId = true;
				} else {
					ids.add(id);
				}
			}
		}

		return removedDuplicateId;
	}

	/**
	 * Returns the next ID not used within the given model.
	 * 
	 * @param existingModel
	 *            the model to be searched
	 * 
	 * @return Next ID not used within the given model.
	 */
	public static int getLargestID(EObject existingModel) {
		int max = 0;
		for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				if(((IIdLabeled)eo).getId() > max) {
					max = ((IIdLabeled)eo).getId();
				}
			}
		}
		return max;
	}

	/**
	 * Generates all IDs of the given model. Already existing IDs are
	 * overwritten.
	 * 
	 * @param existingModel
	 *            the model to be updated
	 */
	public static void generateAllIDs(EObject existingModel) {
		int currentId = 0;
		for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				((IIdLabeled)eo).setId((++currentId));
			}
		}
	}

	/**
	 * Assigns unique IDs to the given object and its content.
	 * 
	 * @param object
	 *            the model element to be prepared
	 * @param existingModel
	 *            the existing model to be considered for used IDs if object
	 *            already contains IDs > 0
	 * @param currentMaxId
	 *            the current maximum ID to be started
	 * @return the new current maximum ID
	 */
	public static int prepareIDs(EObject object, EObject existingModel, int currentMaxId) {
		if(object == null) {
			return currentMaxId;
		}

		boolean needSmart = false;
		if(object instanceof IIdLabeled) {
			if(((IIdLabeled)object).getId() <= 0) {
				((IIdLabeled)object).setId(++currentMaxId);
			} else {
				needSmart = true;
			}
		}

		for(Iterator<EObject> i = object.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				if(((IIdLabeled)eo).getId() <= 0) {
					((IIdLabeled)eo).setId(++currentMaxId);
				} else {
					needSmart = true;
				}
			}
		}

		if(needSmart) {
			Set<Integer> usedIDs = new HashSet<Integer>();
			for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
				EObject eo = i.next();
				if(eo instanceof IIdLabeled) {
					usedIDs.add(((IIdLabeled)eo).getId());
				}
			}
			if(object instanceof IIdLabeled) {

				if(!usedIDs.add(((IIdLabeled)object).getId())) {
					int newId = ++currentMaxId;
					((IIdLabeled)object).setId(newId);
					usedIDs.add(newId);
				}
			}

			for(Iterator<EObject> i = object.eAllContents(); i.hasNext();) {
				EObject eo = i.next();
				if(eo instanceof IIdLabeled) {
					if(!usedIDs.add(((IIdLabeled)eo).getId())) {
						int newId = ++currentMaxId;
						((IIdLabeled)eo).setId(newId);
						usedIDs.add(newId);
					}
				}
			}
		}
		return currentMaxId;
	}

	/**
	 * Assigns unique IDs to the given target model element.
	 * 
	 * @param element
	 *            the model element, which should be prepared for insertion into
	 *            the given persisted model.
	 * @param context
	 *            a model element already contained in a persisted model.
	 * @throws NullPointerException
	 *             if the context is not contained in a persisted model.
	 */
	public static void prepareUniqueID(EObject element, EObject context) {
		if(element instanceof IIdLabeled) {
			ITopLevelElement top = IPersistencyService.getInstance().getTopLevelElementFor(context);

			if(top == null) {
				// Since we do not catch the max id, the id can be directly set as max id + 1
				EObject root = context;
				while(root.eContainer() != null) {
					root = root.eContainer();
				}

				// see also org.fortiss.tooling.kernel.internal.storage.eclipse.ModelContext:129
				int id = 0;
				for(TreeIterator<EObject> iter = root.eAllContents(); iter.hasNext();) {
					EObject o = iter.next();
					if(o instanceof IIdLabeled) {
						id = Math.max(id, ((IIdLabeled)o).getId());
					}
				}

				((IIdLabeled)element).setId(id + 1);
			} else {
				top.prepareIDs(element);
			}
		}
	}

	/**
	 * Verify whether there are missing id.
	 * 
	 * @param existingModel
	 *            a model element representing the scope in which the check is done.
	 * 
	 * @return {@code true} iff the {@code existingModel} has missing IDs.
	 */
	public static boolean hasMissingIDs(EObject existingModel) {
		for(Iterator<EObject> i = existingModel.eAllContents(); i.hasNext();) {
			EObject eo = i.next();
			if(eo instanceof IIdLabeled) {
				if(((IIdLabeled)eo).getId() <= 0) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * Generates all missing IDs of the given model.
	 * 
	 * @param modelElement
	 *            top level element
	 */
	public static void fixMissingIDs(final ITopLevelElement modelElement) {
		final int maxID = getLargestID(modelElement.getRootModelElement());
		if(maxID > 0) {
			modelElement.runAsCommand(new Runnable() {
				@Override
				public void run() {
					generateMissingIDs(modelElement.getRootModelElement(), maxID);
				}
			});
		}
	}
}
