/*--------------------------------------------------------------------------+
$Id: codetemplates.xml 1 2011-01-01 00:00:01Z hoelzl $
|                                                                          |
| Copyright 2016 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.spiderchart.style;

import static org.fortiss.tooling.spiderchart.util.RGBColorUtils.GREEN;
import static org.fortiss.tooling.spiderchart.util.RGBColorUtils.getBrighterColor;

import java.util.HashMap;
import java.util.Map;

import org.fortiss.tooling.spiderchart.model.Axis;
import org.fortiss.tooling.spiderchart.model.DataSeries;
import org.fortiss.tooling.spiderchart.model.SpiderChart;

/**
 * Class for all style attributes of the {@link SpiderChart spider chart}.
 * 
 * @author hoelzl
 * @author $Author: hoelzl $
 * @version $Rev: 18709 $
 * @ConQAT.Rating YELLOW Hash: 236F885B17E9A70F350F85014456AFDF
 */
public final class ChartStyle {
	/** The map for style information of axes. */
	private Map<Axis, AxisStyle> axisStyle = new HashMap<Axis, AxisStyle>();
	/** The map for style information of data series. */
	private Map<DataSeries, DataSeriesStyle> dataSeriesStyle =
			new HashMap<DataSeries, DataSeriesStyle>();
	/** Flag for showing the title. */
	private boolean showTitle = true;
	/** The font style for the title. */
	private FontStyle titleStyle;
	/** Flag for showing the legend. */
	private boolean showLegend = true;
	/** The style for the legend. */
	private LegendStyle legendStyle;
	/** The relative chart margin. The default is 10%. */
	private double relativeMargin = 0.1;
	/** The start angle of the first axis (default is 90 degree). */
	private double startAngleDegree = 90.0;
	/**
	 * The flag for using individual segments on each axis. No background spider web will be drawn
	 * if this is <code>true</code>.
	 */
	private boolean useIndividualAxisSegments = false;
	/**
	 * The number of segments to be drawn if {@link #useIndividualAxisSegments} is
	 * <code>false</code>.
	 */
	private int axisSegments = 1;
	/**
	 * The {@link FillStyle} used for the background if {@link #useIndividualAxisSegments} is
	 * <code>false</code>.
	 */
	private FillStyle backgroundFillStyle = new FillStyle(getBrighterColor(GREEN));
	/**
	 * The {@link LineStyle} used for the background if {@link #useIndividualAxisSegments} is false.
	 */
	private LineStyle backgroundLineStyle = new LineStyle(GREEN);

	/**
	 * Constructor.
	 */
	public ChartStyle(boolean showTitle, boolean showLegend, boolean useIndividualAxisSegments) {
		this.showTitle = showTitle;
		this.showLegend = showLegend;
		this.useIndividualAxisSegments = useIndividualAxisSegments;
	}

	/** Returns the start angle in degree. */
	public double getStartAngleDegree() {
		return startAngleDegree;
	}

	/** Sets the start angle in degree. */
	public void setStartAngleDegree(double startAngleDegree) {
		this.startAngleDegree = startAngleDegree;
	}

	/**
	 * Sets the relative margin of the chart area. Specify <code>0.1</code> for a margin of 10
	 * percent.
	 */
	public void setRelativeMargin(double relativeMargin) {
		this.relativeMargin = relativeMargin;
	}

	/** Returns the relative margin. */
	public double getRelativeMargin() {
		return relativeMargin;
	}

	/** Returns whether to show the title. */
	public boolean isShowTitle() {
		return showTitle;
	}

	/** Sets the showTitle flag. */
	public void setShowTitle(boolean showTitle) {
		this.showTitle = showTitle;
	}

	/** Returns the title font style. */
	public FontStyle getTitleStyle() {
		return titleStyle;
	}

	/** Sets the title font style. */
	public void setTitleStyle(FontStyle titleStyle) {
		this.titleStyle = titleStyle;
	}

	/** Returns whether to show the legend. */
	public boolean isShowLegend() {
		return showLegend;
	}

	/** Sets showLegend flag. */
	public void setShowLegend(boolean showLegend) {
		this.showLegend = showLegend;
	}

	/** Returns the legend style. */
	public LegendStyle getLegendStyle() {
		return legendStyle;
	}

	/** Sets the legend style. */
	public void setLegendStyle(LegendStyle legendStyle) {
		this.legendStyle = legendStyle;
	}

	/** Returns the style for the given axis. */
	public AxisStyle getAxisStyle(Axis axis) {
		return axisStyle.get(axis);
	}

	/** Sets the style for the given axis. */
	public void setAxisStyle(Axis axis, AxisStyle style) {
		axisStyle.put(axis, style);
	}

	/** Returns the style for the given data series. */
	public DataSeriesStyle getDataSeriesStyle(DataSeries dataSeries) {
		return dataSeriesStyle.get(dataSeries);
	}

	/** Sets the style for the given data series. */
	public void setDataSeriesStyle(DataSeries dataSeries, DataSeriesStyle style) {
		dataSeriesStyle.put(dataSeries, style);
	}

	/** Returns whether to use individual axis segments. */
	public boolean isUseIndividualAxisSegments() {
		return useIndividualAxisSegments;
	}

	/** Sets the useIndividualAxisSegments flag. */
	public void setUseIndividualAxisSegments(boolean useIndividualAxisSegments) {
		this.useIndividualAxisSegments = useIndividualAxisSegments;
	}

	/** Returns the axis segments. */
	public int getAxisSegments() {
		return axisSegments;
	}

	/** Sets axis segments. */
	public void setAxisSegments(int axisSegments) {
		this.axisSegments = Math.max(1, axisSegments);
	}

	/** Returns the background fill style. */
	public FillStyle getBackgroundFillStyle() {
		return backgroundFillStyle;
	}

	/** Sets the background fill style. */
	public void setBackgroundFillStyle(FillStyle backgroundFillStyle) {
		this.backgroundFillStyle = backgroundFillStyle;
	}

	/** Returns the background line style. */
	public LineStyle getBackgroundLineStyle() {
		return backgroundLineStyle;
	}

	/** Sets the background line style. */
	public void setBackgroundLineStyle(LineStyle backgroundLineStyle) {
		this.backgroundLineStyle = backgroundLineStyle;
	}
}
