/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.editor;

import java.io.File;

import org.eclipse.core.filesystem.EFS;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.ide.FileStoreEditorInput;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.ITextEditor;

import org.conqat.lib.commons.filesystem.CanonicalFile;
import org.conqat.lib.commons.string.StringUtils;
import org.conqat.ide.commons.ui.dialog.MessageUtils;
import org.conqat.ide.commons.ui.ui.EmptyPartListener;

/**
 * Offers helper methods for editor handling.
 * 
 * This class handles two dimensions: (1) files can be in the workspace or
 * outside, (2) editors can be internal or outside. Only a subset of all
 * possible combinations is currently supported.
 * 
 * @author aswolins
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: F3E1F059B93285235D95CAB6617689FF
 */
public class EditorUtils {

	/**
	 * This method tries to open the file in an internal editor, regardless if
	 * the file is in the workspace or not. If it can't find a default editor,
	 * it returns <code>null</code>.
	 * 
	 * @param file
	 *            The path to the file
	 * @return the TextEditor or <code>null</code>, if no internal editor can be
	 *         found
	 * @throws PartInitException
	 *             if the File couldn't be found or if editor could not be
	 *             created or initialized the editor
	 */
	public static ITextEditor openInInternalEditor(File file)
			throws PartInitException {
		IPath path = new Path(file.getPath());
		IEditorDescriptor editorDescriptor = PlatformUI.getWorkbench()
				.getEditorRegistry().getDefaultEditor(path.lastSegment());
		if (editorDescriptor == null) {
			// no editor associated - we cannot open any
			String message = "No editor is associated with this file extension. "
					+ StringUtils.CR
					+ StringUtils.CR
					+ "Use the preferences (General | Editor | File Associations) to create a new file association"
					+ "to open the file in the internal editor.";
			MessageUtils.showWarningAsync("Cannot open internal editor",
					message);
			return null;
		}

		IFile resource = ResourcesPlugin.getWorkspace().getRoot()
				.getFileForLocation(path);

		IWorkbenchPage workbenchPage = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		if (resource != null) {
			// the file is in in the workspace and an editor is associated
			return (ITextEditor) workbenchPage.openEditor(new FileEditorInput(
					resource), editorDescriptor.getId());
		}

		return openEditorWithEFS(file, editorDescriptor, workbenchPage);
	}

	/**
	 * Opens the file in a external editor. Also works for files that are in the
	 * workspace.
	 * 
	 * @param file
	 *            The path to the file
	 * 
	 * 
	 * @return Returns an {@link ITextEditor} or <code>null</code>, if for some
	 *         reason there is no external editor available
	 * 
	 * @throws PartInitException
	 *             if the File couldn't be found or if editor could not be
	 *             created or initialized the editor
	 */
	public static ITextEditor openInExternalEditor(CanonicalFile file)
			throws PartInitException {

		ITextEditor editor = null;

		IWorkbenchPage workbenchPage = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();

		IEditorDescriptor desc = PlatformUI.getWorkbench().getEditorRegistry()
				.findEditor(IEditorRegistry.SYSTEM_EXTERNAL_EDITOR_ID);

		editor = openEditorWithEFS(file, desc, workbenchPage);

		return editor;
	}

	/**
	 * Opens workspace external files in an editor. The
	 * {@link IEditorDescriptor} defines which editor is to be used and if the
	 * editor is internal or external. The eclipse file system is used to get a
	 * {@link IFileStore} corresponding to the local file system. The editor is
	 * then opened on the {@link IFileStore}.
	 * 
	 * @return the TextEditor
	 * */
	private static ITextEditor openEditorWithEFS(File file,
			IEditorDescriptor desc, IWorkbenchPage workbenchPage)
			throws PartInitException {

		if (!file.canRead()) {
			return null;
		}

		IFileStore fileStore = EFS.getLocalFileSystem().getStore(file.toURI());

		return (ITextEditor) workbenchPage.openEditor(new FileStoreEditorInput(
				fileStore), desc.getId());
	}

	/**
	 * This method hooks in a listener which closes the editor if the given file
	 * is deleted or no longer accessible (due to a closed project). This is
	 * usually called in init(), but the editor site must be already set when
	 * this is called.
	 */
	public static void makeEditorResourceAware(IEditorPart editor,
			final IFile file) {
		final IResourceChangeListener listener = new CloseEditorOnFileRemovedListener(
				editor, file);

		file.getWorkspace().addResourceChangeListener(listener);

		// remove listener when editor no longer exists.
		editor.getEditorSite().getPage().addPartListener(
				new EmptyPartListener() {
					@Override
					public void partClosed(IWorkbenchPart part) {
						file.getWorkspace().removeResourceChangeListener(
								listener);
					}
				});
	}

}