/*-------------------------------------------------------------------------+
|                                                                          |
| Copyright 2005-2011 The ConQAT Project                                   |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+-------------------------------------------------------------------------*/
package org.conqat.ide.commons.ui.xmlmodel.undo;

import java.io.IOException;
import java.io.StringReader;
import java.util.List;

import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.eclipse.jface.action.IAction;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import org.conqat.lib.commons.string.StringUndoStackBase;
import org.conqat.lib.commons.string.StringUtils;
import org.conqat.lib.commons.xml.XMLUtils;
import org.conqat.ide.commons.ui.CCSMCommonsActivator;
import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.conqat.ide.commons.ui.xmlmodel.IModelListener;
import org.conqat.ide.commons.ui.xmlmodel.XmlRootModelBase;

/**
 * A stack storing several versions of an {@link XmlRootModelBase} and providing
 * methods and listeners which are suitable for undo and redo.
 * 
 * @author hummelb
 * @author $Author: deissenb $
 * @version $Rev: 34252 $
 * @levd.rating GREEN Hash: EF03D40D0135F3F0D0E252861612B0C8
 */
public class ModelVersionStack extends StringUndoStackBase implements
		IModelListener {

	/** The model we are versioning. */
	private final XmlRootModelBase model;

	/** Flag for protecting us from dealing with model change events. */
	private boolean ignoreModelChanges = false;

	/** The undo action. */
	private final IAction undoAction = new ModelUndoAction(this);

	/** The redo action. */
	private final IAction redoAction = new ModelRedoAction(this);

	/** Constructor. */
	public ModelVersionStack(XmlRootModelBase model) {
		super(getModelString(model));
		this.model = model;

		// update the actions.
		fireStackChanged();

		// we do never unregister; we will live as long as the model does
		model.addModelListener(this);
	}

	/** Returns the redo action for this stack. */
	public IAction getRedoAction() {
		return redoAction;
	}

	/** Returns the undo action for this stack. */
	public IAction getUndoAction() {
		return undoAction;
	}

	/** {@inheritDoc} */
	public void modelChanged() {
		if (ignoreModelChanges) {
			return;
		}
		insertNewVersion(getModelString(model));
	}

	/** {@inheritDoc} */
	@Override
	protected void fireStackChanged() {
		undoAction.setEnabled(canUndo());
		redoAction.setEnabled(canRedo());
	}

	/** Returns a textual representation of the model. */
	private static String getModelString(XmlRootModelBase model) {
		try {
			return XMLUtils.prettyPrint(model.getDocument());
		} catch (TransformerConfigurationException e) {
			throw new AssertionError(
					"This is not possible (transformer is part of the JDK)");
		} catch (TransformerException e) {
			throw new AssertionError(
					"This is not possible (transformation is sane)");
		}
	}

	/** Puts a textual representation of the model back into it. */
	@Override
	protected void setModelFromString(String modelString) {
		Document document;
		try {
			document = XMLUtils.parse(new InputSource(new StringReader(
					modelString)));
		} catch (IOException e) {
			throw new AssertionError(
					"This is not possible (we are reading from a string)");
		} catch (SAXException e) {
			// As we serialized the string ourselves, this is a programming
			// error.
			LoggingUtils.error(CCSMCommonsActivator.getDefault(),
					"Internal stack error", e);
			return;
		}

		ignoreModelChanges = true;
		model.insertNewDocument(document);
		ignoreModelChanges = false;
	}

	/** {@inheritDoc} */
	@Override
	protected String join(List<String> parts) {
		return StringUtils.concat(parts, StringUtils.CR);
	}

	/** {@inheritDoc} */
	@Override
	protected List<String> split(String s) {
		return StringUtils.splitLinesAsList(s);
	}
}